@@ -13,13 +13,15 @@ import importlib
 
 
 import yaml
-from jinja2 import Environment, FileSystemLoader, TemplateSyntaxError
+from jinja2 import (Environment, FileSystemLoader, TemplateSyntaxError,
+                    Undefined, DebugUndefined, StrictUndefined)
 from mkdocs.plugins import BasePlugin
 from mkdocs.config import config_options
 from mkdocs.config.config_options import Type as PluginType
 
 
-from .util import install_package, parse_package, trace, debug, update, SuperDict, import_local_module, format_chatter, LOG
+from .util import (install_package, parse_package, trace, debug,
+        update, SuperDict, import_local_module, format_chatter, LOG)
 from .context import define_env
 
 # ------------------------------------------
@@ -32,6 +34,30 @@ YAML_VARIABLES = 'extra'
 # The default name of the Python module:
 DEFAULT_MODULE_NAME = 'main'  # main.py
 
+# Possible behavior in case of ignored variables or macros (first is default)
+
+
+class LaxUndefined(Undefined):
+    "Pass anything wrong as blank"
+
+    def _fail_with_undefined_error(self, *args, **kwargs):
+        return ''
+
+
+UNDEFINED_BEHAVIOR = {'keep': DebugUndefined,
+                      'silent': Undefined,
+                      'strict': StrictUndefined,
+                      # lax will even pass unknown objects:
+                      'lax': LaxUndefined}
+
+# By default undefined jinja2 variables AND macros will be left as-is
+# see https://stackoverflow.com/a/53134416
+DEFAULT_UNDEFINED_BEHAVIOR = 'keep'
+
+# Return codes in case of error
+ERROR_MACRO = 100
+
+
 # ------------------------------------------
 # Plugin
 # ------------------------------------------
@@ -65,6 +91,10 @@ class MacrosPlugin(BasePlugin):
         ('j2_block_end_string',      J2_STRING),
         ('j2_variable_start_string', J2_STRING),
         ('j2_variable_end_string',   J2_STRING),
+        # for behavior of unknown macro (e.g. other plugin):
+        ('on_undefined',  PluginType(str, default=DEFAULT_UNDEFINED_BEHAVIOR)),
+        # for CD/CI set that parameter to true
+        ('on_error_fail', PluginType(bool, default=False)),
         ('verbose', PluginType(bool, default=False))
     )
 
@@ -225,28 +255,17 @@ class MacrosPlugin(BasePlugin):
             raise AttributeError("Too early: raw markdown is not available"
                                  "at this stage!")
 
-<<<<<<< HEAD
     @raw_markdown.setter
-    def raw_markdown(self, s):
+    def raw_markdown(self, value):
         """
         Used to set the raw markdown
         """
-        if not isinstance(s, str):
+        if not isinstance(value, str):
             raise ValueError("Value provided to attribute raw_markdown "
                              "should be a string")
+        # check whether attribute is accessible:
         self.raw_markdown
-        self._raw_markdown = s
-=======
-
-    @raw_markdown.setter
-    def raw_markdown(self, value):
-        try:
-            _ = self._raw_markdown
-        except AttributeError:
-            raise AttributeError("Too early: raw markdown is not available"
-                                 "at this stage!")
         self._raw_markdown = value
->>>>>>> a33296c16876c1e52a2c79c9fb3d0beff53c38ad
 
     # ----------------------------------
     # Function lists, for later events
@@ -454,6 +473,7 @@ class MacrosPlugin(BasePlugin):
             page_variables.update(meta_variables)
 
         # expand the template
+        on_error_fail = self.config['on_error_fail']
         try:
             md_template = self.env.from_string(markdown)
             # Execute the jinja2 template and return
@@ -468,7 +488,11 @@ class MacrosPlugin(BasePlugin):
                       "```"]
             error = "\n".join(output)
             trace("ERROR", error)
-            return error
+            if on_error_fail:
+                exit(ERROR_MACRO)
+            else:
+                # default case
+                return error
         except Exception as e:
             output = ["# _Macro Rendering Error_",
                       "",
@@ -479,7 +503,11 @@ class MacrosPlugin(BasePlugin):
                       "```"]
             error = "\n".join(output)
             trace("ERROR", error)
-            return error
+            if on_error_fail:
+                exit(ERROR_MACRO)
+            else:
+                # default case
+                return error
 
     # ----------------------------------
     # Standard Hooks for a mkdocs plugin
@@ -550,9 +578,15 @@ class MacrosPlugin(BasePlugin):
             trace("Includes directory:", include_dir)
         else:
             debug("Includes directory:", include_dir)
-        # will contain all parameters:
+        # get the behavior in case of unknown variable (default: keep)
+        on_undefined = self.config['on_undefined']
+        if on_undefined not in UNDEFINED_BEHAVIOR:
+            raise ValueError("Illegal value for undefined macro parameter '%s'" % on_undefined)
+        undefined = UNDEFINED_BEHAVIOR[on_undefined]
+        debug("Undefined behavior:", undefined)
         env_config = {
-            'loader': FileSystemLoader(include_dir)
+            'loader': FileSystemLoader(include_dir),
+            'undefined': undefined
         }
         # read the config variables for jinja2:
         for key, value in self.config.items():
